/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

/**
 * Title:        Quake3Arena MasterBrowser
 * Description:
 * Copyright:    Copyright (c) 2001 by Hendrik Lipka
 * @author Hendrik Lipka
 * @version 0.1
*/

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.text.*;

import org.log4j.*;

public class StatusHandler
{
    private Category CAT=Category.getInstance(StatusHandler.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.status");

    protected Hashtable _servers=new Hashtable();

    protected long _timeout=1000*60*11; // 11 minutes - heartbeat is sent every 5 minutes

    protected DateFormat _format=DateFormat.getTimeInstance(DateFormat.MEDIUM);

    protected static ServerStatusParser _statusParser=new ServerStatusParser();

    protected Configuration _config=null;

    public void StatusHandler()
    {
        _timeout=Configuration.getInstance().getServerTimeout();
        _config=Configuration.getInstance();
    }

    /**
     * @todo query server to get more informations
    */
    public void addServer(ServerData data)
    {
        boolean state=_statusParser.queryServerStatus(data);
        if (state)
        {
            _servers.put(data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort),data);
            LOG_CAT.info(" - adding server with: "+data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
        }
        else
        {
            _servers.remove(data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
            LOG_CAT.info(" - removing server with: "+data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
        }
    }

    public ServerData[] getAllServers()
    {
        return getAllServers(-1,true,true);
    }

    public ServerData[] getAllServers(int protocol)
    {
        return getAllServers(protocol,true,true);
    }

    public ServerData[] getAllServers(boolean full, boolean empty)
    {
        return getAllServers(-1,full,empty);
    }

    /**
     * @todo add check for timeout
     * @todo add check for protocol, full and empty servers
    */
    public ServerData[] getAllServers(int protocol, boolean full, boolean empty)
    {
        long time=System.currentTimeMillis();

        Enumeration keys=_servers.keys();
        Vector v=new Vector();
        while (keys.hasMoreElements())
        {
            Object o=keys.nextElement();
            ServerData sd=(ServerData)_servers.get(o);
            // we need to check for null here, because other threads might remove timed out servers...
            if (null!=o)
            {
                // check for timeout
                if ((time-sd._lastHeartBeat)>_timeout)
                {
                    LOG_CAT.info("removing server with key "+(String)o);
                    _servers.remove(o);
                    continue;
                }
                // filter
                CAT.debug("server="+sd._serverAddress+":"+sd._serverPort);
                _config=Configuration.getInstance();
                if (!full&&(sd._players==sd._maxPlayers)&&_config.filterFull())
                {
                    CAT.debug("filtered because full");
                    continue;
                }
                if (!empty&&(0==sd._players)&&_config.filterEmpty())
                {
                    CAT.debug("filtered because empty");
                    continue;
                }
                if ((-1!=protocol)&&(protocol!=sd._protocol)&&_config.filterProtocol())
                {
                    CAT.debug("filtered because wrong protocol");
                    continue;
                }
                // add server to result
                CAT.debug("added server <"+sd._serverAddress+":"+sd._serverPort+"> to result");
                v.addElement(sd);
            }
        }

        if (0==v.size())
            return null;

        ServerData[] sda=new ServerData[v.size()];
        for (int i=0;i<sda.length;i++)
        {
            sda[i]=(ServerData)v.elementAt(i);
        }
        return sda;
    }
}